var SRTM = ee.Image("USGS/SRTMGL1_003");

//////////////////////// regions uploaded based on the subwatershed ////////////////////////
var studyArea =ee.FeatureCollection('users/river extraction/LCJ'); 

// // decide the region and the filenames
var polygon = studyArea;
var fname = 'studyArea'; 

Map.addLayer(polygon, {color: "red"}, fname, true);
Map.centerObject(polygon, 4);

/////////////////////////////////////////////////////////////////////////////
// Composite time period
// // // Time period
var Filter1 = ee.Filter.date('2017-08-24','2017-08-25');
var Filter2 = ee.Filter.date('2018-09-18','2018-09-19');
var Filter3 = ee.Filter.date('2018-09-16','2018-09-17');

// composite date
var dateFilter = ee.Filter.or(Filter1, Filter2, Filter3);

// /////////////////////////////////////////////////////////////////////////////
// First we use the Sentinel-2 QA band to cloud mask the collection.
// // Function to mask clouds using the Sentinel-2 QA band.
function maskS2clouds(image) {
  var qa = image.select('QA60');

  // Bits 10 and 11 are clouds and cirrus, respectively.
  var cloudBitMask = 1 << 10;
  var cirrusBitMask = 1 << 11;

  // Both flags should be set to zero, indicating clear conditions.
  var mask = qa.bitwiseAnd(cloudBitMask).eq(0).and(
            qa.bitwiseAnd(cirrusBitMask).eq(0));

  // Return the masked and scaled data, without the QA bands.
  return image.updateMask(mask)//.divide(10000)
      .select("B.*")
      .copyProperties(image, ["system:time_start"]);
}

// Choose images according to the dateFilter date and take the median.
// Load Sentinel-2 TOA reflectance data.
var S2C = ee.ImageCollection('COPERNICUS/S2')
    .filter(dateFilter)
    .filterBounds(polygon)
    // Pre-filter to get less cloudy granules.
    .filter(ee.Filter.lt('CLOUDY_PIXEL_PERCENTAGE', 30))
    .map(maskS2clouds);
var median = ee.Image(S2C.median());

// Select the red, green and blue bands.
var result = median.select('B4', 'B3', 'B2');
Map.addLayer(result.clip(polygon), {gain: '0.1, 0.1, 0.1', scale:20});
Map.centerObject(polygon, 4);

// //dem
var dem = ee.Image(SRTM).clip(polygon);
// //slope
var slope = ee.Terrain.slope(dem).clip(polygon);

// // // // //// water index calculation    ///////////////////
var mndwi = median.normalizedDifference(['B3', 'B11']).rename("MNDWI");

////histogram drawing and calculation
print(ui.Chart.image.histogram({
  image: mndwi,
  region: polygon,
  scale:500
}));

// histogram calculation
var mndwiHistogram = mndwi.reduceRegion({
  reducer: ee.Reducer.histogram(), 
  geometry: polygon, 
  scale: 10,
  maxPixels: 1e13
});

// // ////Otsu Segmentation algorithm
var otsu = function(histogram) {
  var counts = ee.Array(ee.Dictionary(histogram).get('histogram'));
  var means = ee.Array(ee.Dictionary(histogram).get('bucketMeans'));
  var size = means.length().get([0]);
  var total = counts.reduce(ee.Reducer.sum(), [0]).get([0]);
  var sum = means.multiply(counts).reduce(ee.Reducer.sum(), [0]).get([0]);
  var mean = sum.divide(total);
  
  var indices = ee.List.sequence(1, size);
  
  // Compute between sum of squares, where each mean partitions the data.
  var bss = indices.map(function(i) {
    var aCounts = counts.slice(0, 0, i);
    var aCount = aCounts.reduce(ee.Reducer.sum(), [0]).get([0]);
    var aMeans = means.slice(0, 0, i);
    var aMean = aMeans.multiply(aCounts)
        .reduce(ee.Reducer.sum(), [0]).get([0])
        .divide(aCount);
    var bCount = total.subtract(aCount);
    var bMean = sum.subtract(aCount.multiply(aMean)).divide(bCount);
    return aCount.multiply(aMean.subtract(mean).pow(2)).add(
          bCount.multiply(bMean.subtract(mean).pow(2)));
  });
  // print(ui.Chart.array.values(ee.Array(bss), 0, means));
  // Return the mean value corresponding to the maximum BSS.
  return means.sort(bss).get([-1]);
};

// //Segmentation threshold calculation
var mndwiThreshold = otsu(ee.Number(mndwiHistogram.get("MNDWI")));
print("mndwiThreshold: ", mndwiThreshold);

// Determine the final export result based on mndwiThreshold and visual inspection
Map.addLayer(mndwi.clip(polygon)
  .updateMask(mndwi.gt(mndwiThreshold)), 
  {palette: "yellow"}, 
  "MNDWI",
  true
);

